# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["DetailRecordsReportListParams"]


class DetailRecordsReportListParams(TypedDict, total=False):
    page_number: Annotated[int, PropertyInfo(alias="page[number]")]
    """The page number to load."""

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]
    """The size of the page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ActionRecordPauseParams"]


class ActionRecordPauseParams(TypedDict, total=False):
    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    recording_id: str
    """Use this field to pause specific recording."""

    region: Literal["Australia", "Europe", "Middle East", "US"]
    """Region where the conference data is located.

    Defaults to the region defined in user's data locality settings (Europe or US).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .aws_voice_settings_param import AwsVoiceSettingsParam
from .transcription_config_param import TranscriptionConfigParam
from .interruption_settings_param import InterruptionSettingsParam
from .telnyx_voice_settings_param import TelnyxVoiceSettingsParam
from .eleven_labs_voice_settings_param import ElevenLabsVoiceSettingsParam
from ..shared_params.rime_voice_settings import RimeVoiceSettings
from ..shared_params.azure_voice_settings import AzureVoiceSettings
from ..shared_params.resemble_voice_settings import ResembleVoiceSettings

__all__ = [
    "ActionStartAIAssistantParams",
    "Assistant",
    "MessageHistory",
    "MessageHistoryUserMessage",
    "MessageHistoryAssistantMessage",
    "MessageHistoryAssistantMessageToolCall",
    "MessageHistoryAssistantMessageToolCallFunction",
    "MessageHistoryToolMessage",
    "MessageHistorySystemMessage",
    "MessageHistoryDeveloperMessage",
    "Participant",
    "VoiceSettings",
]


class ActionStartAIAssistantParams(TypedDict, total=False):
    assistant: Assistant
    """AI Assistant configuration"""

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    greeting: str
    """
    Text that will be played when the assistant starts, if none then nothing will be
    played when the assistant starts. The greeting can be text for any voice or SSML
    for `AWS.Polly.<voice_id>` voices. There is a 3,000 character limit.
    """

    interruption_settings: InterruptionSettingsParam
    """Settings for handling user interruptions during assistant speech"""

    message_history: Iterable[MessageHistory]
    """A list of messages to seed the conversation history before the assistant starts.

    Follows the same message format as the `ai_assistant_add_messages` command.
    """

    participants: Iterable[Participant]
    """A list of participants to add to the conversation when it starts."""

    send_message_history_updates: bool
    """
    When `true`, a webhook is sent each time the conversation message history is
    updated.
    """

    transcription: TranscriptionConfigParam
    """The settings associated with speech to text for the voice assistant.

    This is only relevant if the assistant uses a text-to-text language model. Any
    assistant using a model with native audio support (e.g.
    `fixie-ai/ultravox-v0_4`) will ignore this field.
    """

    voice: str
    """The voice to be used by the voice assistant.

    Currently we support ElevenLabs, Telnyx and AWS voices.

    **Supported Providers:**

    - **AWS:** Use `AWS.Polly.<VoiceId>` (e.g., `AWS.Polly.Joanna`). For neural
      voices, which provide more realistic, human-like speech, append `-Neural` to
      the `VoiceId` (e.g., `AWS.Polly.Joanna-Neural`). Check the
      [available voices](https://docs.aws.amazon.com/polly/latest/dg/available-voices.html)
      for compatibility.
    - **Azure:** Use `Azure.<VoiceId>. (e.g. Azure.en-CA-ClaraNeural,
      Azure.en-CA-LiamNeural, Azure.en-US-BrianMultilingualNeural,
      Azure.en-US-Ava:DragonHDLatestNeural. For a complete list of voices, go to
      [Azure Voice Gallery](https://speech.microsoft.com/portal/voicegallery).)
    - **ElevenLabs:** Use `ElevenLabs.<ModelId>.<VoiceId>` (e.g.,
      `ElevenLabs.BaseModel.John`). The `ModelId` part is optional. To use
      ElevenLabs, you must provide your ElevenLabs API key as an integration secret
      under `"voice_settings": {"api_key_ref": "<secret_id>"}`. See
      [integration secrets documentation](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
      for details. Check
      [available voices](https://elevenlabs.io/docs/api-reference/get-voices).
    - **Telnyx:** Use `Telnyx.<model_id>.<voice_id>`
    - **Inworld:** Use `Inworld.<ModelId>.<VoiceId>` (e.g., `Inworld.Mini.Loretta`,
      `Inworld.Max.Oliver`). Supported models: `Mini`, `Max`.
    """

    voice_settings: VoiceSettings
    """The settings associated with the voice selected"""


class Assistant(TypedDict, total=False):
    """AI Assistant configuration"""

    id: str
    """The identifier of the AI assistant to use"""

    instructions: str
    """
    The system instructions that the voice assistant uses during the start assistant
    command. This will overwrite the instructions set in the assistant
    configuration.
    """

    openai_api_key_ref: str
    """Reference to the OpenAI API key. Required only when using OpenAI models"""


class MessageHistoryUserMessage(TypedDict, total=False):
    """Messages sent by an end user"""

    content: Required[str]
    """The contents of the user message."""

    role: Required[Literal["user"]]
    """The role of the messages author, in this case `user`."""

    metadata: Dict[str, object]
    """Metadata to add to the message"""


class MessageHistoryAssistantMessageToolCallFunction(TypedDict, total=False):
    """The function that the model called."""

    name: Required[str]
    """The name of the function to call."""


class MessageHistoryAssistantMessageToolCall(TypedDict, total=False):
    """A call to a function tool created by the model."""

    id: Required[str]
    """The ID of the tool call."""

    function: Required[MessageHistoryAssistantMessageToolCallFunction]
    """The function that the model called."""

    type: Required[Literal["function"]]
    """The type of the tool. Currently, only `function` is supported."""


class MessageHistoryAssistantMessage(TypedDict, total=False):
    """Messages sent by the model in response to user messages."""

    role: Required[Literal["assistant"]]
    """The role of the messages author, in this case `assistant`."""

    content: str
    """The contents of the assistant message. Required unless `tool_calls`"""

    metadata: Dict[str, object]
    """Metadata to add to the message"""

    tool_calls: Iterable[MessageHistoryAssistantMessageToolCall]
    """The tool calls generated by the model, such as function calls."""


class MessageHistoryToolMessage(TypedDict, total=False):
    content: Required[str]
    """The contents of the tool message."""

    role: Required[Literal["tool"]]
    """The role of the messages author, in this case `tool`."""

    tool_call_id: Required[str]
    """Tool call that this message is responding to."""

    metadata: Dict[str, object]
    """Metadata to add to the message"""


class MessageHistorySystemMessage(TypedDict, total=False):
    """
    Developer-provided instructions that the model should follow, regardless of messages sent by the user.
    """

    content: Required[str]
    """The contents of the system message."""

    role: Required[Literal["system"]]
    """The role of the messages author, in this case `system`."""

    metadata: Dict[str, object]
    """Metadata to add to the message"""


class MessageHistoryDeveloperMessage(TypedDict, total=False):
    """
    Developer-provided instructions that the model should follow, regardless of messages sent by the user.
    """

    content: Required[str]
    """The contents of the developer message."""

    role: Required[Literal["developer"]]
    """The role of the messages author, in this case developer."""

    metadata: Dict[str, object]
    """Metadata to add to the message"""


MessageHistory: TypeAlias = Union[
    MessageHistoryUserMessage,
    MessageHistoryAssistantMessage,
    MessageHistoryToolMessage,
    MessageHistorySystemMessage,
    MessageHistoryDeveloperMessage,
]


class Participant(TypedDict, total=False):
    id: Required[str]
    """The call_control_id of the participant to add to the conversation."""

    role: Required[Literal["user"]]
    """The role of the participant in the conversation."""

    name: str
    """Display name for the participant."""

    on_hangup: Literal["continue_conversation", "end_conversation"]
    """Determines what happens to the conversation when this participant hangs up."""


VoiceSettings: TypeAlias = Union[
    ElevenLabsVoiceSettingsParam,
    TelnyxVoiceSettingsParam,
    AwsVoiceSettingsParam,
    AzureVoiceSettings,
    RimeVoiceSettings,
    ResembleVoiceSettings,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .phone_number_list_params import PhoneNumberListParams as PhoneNumberListParams
from .phone_number_list_response import PhoneNumberListResponse as PhoneNumberListResponse
from .phone_number_verify_params import PhoneNumberVerifyParams as PhoneNumberVerifyParams
from .business_account_list_params import BusinessAccountListParams as BusinessAccountListParams
from .message_template_list_params import MessageTemplateListParams as MessageTemplateListParams
from .business_account_list_response import BusinessAccountListResponse as BusinessAccountListResponse
from .message_template_create_params import MessageTemplateCreateParams as MessageTemplateCreateParams
from .message_template_update_params import MessageTemplateUpdateParams as MessageTemplateUpdateParams
from .message_template_create_response import MessageTemplateCreateResponse as MessageTemplateCreateResponse
from .message_template_update_response import MessageTemplateUpdateResponse as MessageTemplateUpdateResponse
from .business_account_retrieve_response import BusinessAccountRetrieveResponse as BusinessAccountRetrieveResponse
from .message_template_retrieve_response import MessageTemplateRetrieveResponse as MessageTemplateRetrieveResponse
from .phone_number_resend_verification_params import (
    PhoneNumberResendVerificationParams as PhoneNumberResendVerificationParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["WhatsappReactionParam"]


class WhatsappReactionParam(TypedDict, total=False):
    emoji: str

    message_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.porting_orders import associated_phone_number_list_params, associated_phone_number_create_params
from ...types.porting_orders.porting_associated_phone_number import PortingAssociatedPhoneNumber
from ...types.porting_orders.associated_phone_number_create_response import AssociatedPhoneNumberCreateResponse
from ...types.porting_orders.associated_phone_number_delete_response import AssociatedPhoneNumberDeleteResponse

__all__ = ["AssociatedPhoneNumbersResource", "AsyncAssociatedPhoneNumbersResource"]


class AssociatedPhoneNumbersResource(SyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> AssociatedPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AssociatedPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AssociatedPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AssociatedPhoneNumbersResourceWithStreamingResponse(self)

    def create(
        self,
        porting_order_id: str,
        *,
        action: Literal["keep", "disconnect"],
        phone_number_range: associated_phone_number_create_params.PhoneNumberRange,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssociatedPhoneNumberCreateResponse:
        """Creates a new associated phone number for a porting order.

        This is used for
        partial porting in GB to specify which phone numbers should be kept or
        disconnected.

        Args:
          action: Specifies the action to take with this phone number during partial porting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        return self._post(
            path_template(
                "/porting_orders/{porting_order_id}/associated_phone_numbers", porting_order_id=porting_order_id
            ),
            body=maybe_transform(
                {
                    "action": action,
                    "phone_number_range": phone_number_range,
                },
                associated_phone_number_create_params.AssociatedPhoneNumberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssociatedPhoneNumberCreateResponse,
        )

    def list(
        self,
        porting_order_id: str,
        *,
        filter: associated_phone_number_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: associated_phone_number_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PortingAssociatedPhoneNumber]:
        """Returns a list of all associated phone numbers for a porting order.

        Associated
        phone numbers are used for partial porting in GB to specify which phone numbers
        should be kept or disconnected.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[phone_number], filter[action]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        return self._get_api_list(
            path_template(
                "/porting_orders/{porting_order_id}/associated_phone_numbers", porting_order_id=porting_order_id
            ),
            page=SyncDefaultFlatPagination[PortingAssociatedPhoneNumber],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    associated_phone_number_list_params.AssociatedPhoneNumberListParams,
                ),
            ),
            model=PortingAssociatedPhoneNumber,
        )

    def delete(
        self,
        id: str,
        *,
        porting_order_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssociatedPhoneNumberDeleteResponse:
        """
        Deletes an associated phone number from a porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template(
                "/porting_orders/{porting_order_id}/associated_phone_numbers/{id}",
                porting_order_id=porting_order_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssociatedPhoneNumberDeleteResponse,
        )


class AsyncAssociatedPhoneNumbersResource(AsyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> AsyncAssociatedPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAssociatedPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAssociatedPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAssociatedPhoneNumbersResourceWithStreamingResponse(self)

    async def create(
        self,
        porting_order_id: str,
        *,
        action: Literal["keep", "disconnect"],
        phone_number_range: associated_phone_number_create_params.PhoneNumberRange,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssociatedPhoneNumberCreateResponse:
        """Creates a new associated phone number for a porting order.

        This is used for
        partial porting in GB to specify which phone numbers should be kept or
        disconnected.

        Args:
          action: Specifies the action to take with this phone number during partial porting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        return await self._post(
            path_template(
                "/porting_orders/{porting_order_id}/associated_phone_numbers", porting_order_id=porting_order_id
            ),
            body=await async_maybe_transform(
                {
                    "action": action,
                    "phone_number_range": phone_number_range,
                },
                associated_phone_number_create_params.AssociatedPhoneNumberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssociatedPhoneNumberCreateResponse,
        )

    def list(
        self,
        porting_order_id: str,
        *,
        filter: associated_phone_number_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: associated_phone_number_list_params.Sort | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PortingAssociatedPhoneNumber, AsyncDefaultFlatPagination[PortingAssociatedPhoneNumber]]:
        """Returns a list of all associated phone numbers for a porting order.

        Associated
        phone numbers are used for partial porting in GB to specify which phone numbers
        should be kept or disconnected.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[phone_number], filter[action]

          sort: Consolidated sort parameter (deepObject style). Originally: sort[value]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        return self._get_api_list(
            path_template(
                "/porting_orders/{porting_order_id}/associated_phone_numbers", porting_order_id=porting_order_id
            ),
            page=AsyncDefaultFlatPagination[PortingAssociatedPhoneNumber],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    associated_phone_number_list_params.AssociatedPhoneNumberListParams,
                ),
            ),
            model=PortingAssociatedPhoneNumber,
        )

    async def delete(
        self,
        id: str,
        *,
        porting_order_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssociatedPhoneNumberDeleteResponse:
        """
        Deletes an associated phone number from a porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not porting_order_id:
            raise ValueError(f"Expected a non-empty value for `porting_order_id` but received {porting_order_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template(
                "/porting_orders/{porting_order_id}/associated_phone_numbers/{id}",
                porting_order_id=porting_order_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssociatedPhoneNumberDeleteResponse,
        )


class AssociatedPhoneNumbersResourceWithRawResponse:
    def __init__(self, associated_phone_numbers: AssociatedPhoneNumbersResource) -> None:
        self._associated_phone_numbers = associated_phone_numbers

        self.create = to_raw_response_wrapper(
            associated_phone_numbers.create,
        )
        self.list = to_raw_response_wrapper(
            associated_phone_numbers.list,
        )
        self.delete = to_raw_response_wrapper(
            associated_phone_numbers.delete,
        )


class AsyncAssociatedPhoneNumbersResourceWithRawResponse:
    def __init__(self, associated_phone_numbers: AsyncAssociatedPhoneNumbersResource) -> None:
        self._associated_phone_numbers = associated_phone_numbers

        self.create = async_to_raw_response_wrapper(
            associated_phone_numbers.create,
        )
        self.list = async_to_raw_response_wrapper(
            associated_phone_numbers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            associated_phone_numbers.delete,
        )


class AssociatedPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, associated_phone_numbers: AssociatedPhoneNumbersResource) -> None:
        self._associated_phone_numbers = associated_phone_numbers

        self.create = to_streamed_response_wrapper(
            associated_phone_numbers.create,
        )
        self.list = to_streamed_response_wrapper(
            associated_phone_numbers.list,
        )
        self.delete = to_streamed_response_wrapper(
            associated_phone_numbers.delete,
        )


class AsyncAssociatedPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, associated_phone_numbers: AsyncAssociatedPhoneNumbersResource) -> None:
        self._associated_phone_numbers = associated_phone_numbers

        self.create = async_to_streamed_response_wrapper(
            associated_phone_numbers.create,
        )
        self.list = async_to_streamed_response_wrapper(
            associated_phone_numbers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            associated_phone_numbers.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .credit_account_settle_payment_params import CreditAccountSettlePaymentParams as CreditAccountSettlePaymentParams
from .credit_account_settle_payment_response import (
    CreditAccountSettlePaymentResponse as CreditAccountSettlePaymentResponse,
)
from .credit_account_create_payment_quote_params import (
    CreditAccountCreatePaymentQuoteParams as CreditAccountCreatePaymentQuoteParams,
)
from .credit_account_create_payment_quote_response import (
    CreditAccountCreatePaymentQuoteResponse as CreditAccountCreatePaymentQuoteResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AccessIPRangeCreateParams"]


class AccessIPRangeCreateParams(TypedDict, total=False):
    cidr_block: Required[str]

    description: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import AvailablePhoneNumberListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAvailablePhoneNumbers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        available_phone_number = client.available_phone_numbers.list()
        assert_matches_type(AvailablePhoneNumberListResponse, available_phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        available_phone_number = client.available_phone_numbers.list(
            filter={
                "administrative_area": "administrative_area",
                "best_effort": True,
                "country_code": "country_code",
                "exclude_held_numbers": True,
                "features": ["sms"],
                "limit": 0,
                "locality": "locality",
                "national_destination_code": "national_destination_code",
                "phone_number": {
                    "contains": "contains",
                    "ends_with": "ends_with",
                    "starts_with": "starts_with",
                },
                "phone_number_type": "local",
                "quickship": True,
                "rate_center": "rate_center",
                "reservable": True,
            },
        )
        assert_matches_type(AvailablePhoneNumberListResponse, available_phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.available_phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        available_phone_number = response.parse()
        assert_matches_type(AvailablePhoneNumberListResponse, available_phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.available_phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            available_phone_number = response.parse()
            assert_matches_type(AvailablePhoneNumberListResponse, available_phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAvailablePhoneNumbers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        available_phone_number = await async_client.available_phone_numbers.list()
        assert_matches_type(AvailablePhoneNumberListResponse, available_phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        available_phone_number = await async_client.available_phone_numbers.list(
            filter={
                "administrative_area": "administrative_area",
                "best_effort": True,
                "country_code": "country_code",
                "exclude_held_numbers": True,
                "features": ["sms"],
                "limit": 0,
                "locality": "locality",
                "national_destination_code": "national_destination_code",
                "phone_number": {
                    "contains": "contains",
                    "ends_with": "ends_with",
                    "starts_with": "starts_with",
                },
                "phone_number_type": "local",
                "quickship": True,
                "rate_center": "rate_center",
                "reservable": True,
            },
        )
        assert_matches_type(AvailablePhoneNumberListResponse, available_phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.available_phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        available_phone_number = await response.parse()
        assert_matches_type(AvailablePhoneNumberListResponse, available_phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.available_phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            available_phone_number = await response.parse()
            assert_matches_type(AvailablePhoneNumberListResponse, available_phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import number_lookup_retrieve_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.number_lookup_retrieve_response import NumberLookupRetrieveResponse

__all__ = ["NumberLookupResource", "AsyncNumberLookupResource"]


class NumberLookupResource(SyncAPIResource):
    """Look up phone number data"""

    @cached_property
    def with_raw_response(self) -> NumberLookupResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return NumberLookupResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NumberLookupResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return NumberLookupResourceWithStreamingResponse(self)

    def retrieve(
        self,
        phone_number: str,
        *,
        type: Literal["carrier", "caller-name"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberLookupRetrieveResponse:
        """
        Returns information about the provided phone number.

        Args:
          type: Specifies the type of number lookup to be performed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._get(
            path_template("/number_lookup/{phone_number}", phone_number=phone_number),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"type": type}, number_lookup_retrieve_params.NumberLookupRetrieveParams),
            ),
            cast_to=NumberLookupRetrieveResponse,
        )


class AsyncNumberLookupResource(AsyncAPIResource):
    """Look up phone number data"""

    @cached_property
    def with_raw_response(self) -> AsyncNumberLookupResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNumberLookupResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNumberLookupResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncNumberLookupResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        phone_number: str,
        *,
        type: Literal["carrier", "caller-name"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberLookupRetrieveResponse:
        """
        Returns information about the provided phone number.

        Args:
          type: Specifies the type of number lookup to be performed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._get(
            path_template("/number_lookup/{phone_number}", phone_number=phone_number),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"type": type}, number_lookup_retrieve_params.NumberLookupRetrieveParams
                ),
            ),
            cast_to=NumberLookupRetrieveResponse,
        )


class NumberLookupResourceWithRawResponse:
    def __init__(self, number_lookup: NumberLookupResource) -> None:
        self._number_lookup = number_lookup

        self.retrieve = to_raw_response_wrapper(
            number_lookup.retrieve,
        )


class AsyncNumberLookupResourceWithRawResponse:
    def __init__(self, number_lookup: AsyncNumberLookupResource) -> None:
        self._number_lookup = number_lookup

        self.retrieve = async_to_raw_response_wrapper(
            number_lookup.retrieve,
        )


class NumberLookupResourceWithStreamingResponse:
    def __init__(self, number_lookup: NumberLookupResource) -> None:
        self._number_lookup = number_lookup

        self.retrieve = to_streamed_response_wrapper(
            number_lookup.retrieve,
        )


class AsyncNumberLookupResourceWithStreamingResponse:
    def __init__(self, number_lookup: AsyncNumberLookupResource) -> None:
        self._number_lookup = number_lookup

        self.retrieve = async_to_streamed_response_wrapper(
            number_lookup.retrieve,
        )
